/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.core.chart;

import java.util.List;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.office.filter.api.OCKey;
import com.openexchange.office.filter.api.OCValue;

public class ChartAxis extends AttributeSet {

    private final AttributeSet grid = new AttributeSet();
    private final AttributeSet title = new AttributeSet();

    private Integer axis;

    ChartAxis() {

    }

    public void setAxis(Integer axis) {
        this.axis = axis;
    }

    public void createInsertOperation(List<Integer> start, JSONArray operationQueue) throws JSONException {

        if (!getAttributes().isEmpty()) {
            String axPos = null;
            Integer crossAx = null;
            if (axis == 0) {
                axPos = "b";
                crossAx = 1;
            } else if (axis == 1) {
                axPos = "l";
                crossAx = 0;
            } else if (axis == 2) {
                axPos = "t";
                crossAx = 3;
            } else if (axis == 3){
                axPos = "r";
                crossAx = 2;
            } else if (axis == 10) {
                axPos = "b";
                crossAx = 1;
            } else if (axis == 11) {
                axPos = "t";
                crossAx = 3;
            }
            
            JSONObject op = new JSONObject(7);
            op.put(OCKey.START.value(), start);
            op.putSafe(OCKey.AXIS.value(), axis);
            op.putSafe(OCKey.ATTRS.value(), getAttributes());
            op.putSafe(OCKey.NAME.value(), OCValue.SET_CHART_AXIS_ATTRIBUTES.value());
            op.putSafe(OCKey.CROSS_AX.value(), crossAx);
            op.putSafe(OCKey.AX_POS.value(), axPos);
            if (axis == 10 || axis == 11) {
                op.putSafe(OCKey.Z_AXIS.value(), true);
            }
            operationQueue.put(op);
        }

        if (!grid.getAttributes().isEmpty()) {
            JSONObject op = new JSONObject(4);
            op.put(OCKey.START.value(), start);
            op.putSafe(OCKey.AXIS.value(), axis);
            op.putSafe(OCKey.ATTRS.value(), grid.getAttributes());
            op.putSafe(OCKey.NAME.value(), OCValue.SET_CHART_GRIDLINE_ATTRIBUTES.value());
            operationQueue.put(op);
        }

        if (!title.getAttributes().isEmpty()) {
            JSONObject op = new JSONObject(4);
            op.put(OCKey.START.value(), start);
            op.putSafe(OCKey.AXIS.value(), axis);
            op.putSafe(OCKey.ATTRS.value(), title.getAttributes());
            op.putSafe(OCKey.NAME.value(), OCValue.SET_CHART_TITLE_ATTRIBUTES.value());
            operationQueue.put(op);
        }

    }

    public Integer getAxis() {
        return axis;
    }

    public AttributeSet getTitle() {
        return title;
    }

    public AttributeSet getGridLine() {
        return grid;
    }

}
