/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.ods.dom;

import org.xml.sax.Attributes;
import com.openexchange.office.filter.odf.ElementNS;
import com.openexchange.office.filter.odf.SaxContextHandler;
import com.openexchange.office.filter.odf.UnknownContentHandler;

public class ContentValidationHandler extends SaxContextHandler {

	final private ContentValidation contentValidation;
	final private ContentValidations contentValidations;

	public ContentValidationHandler(SaxContextHandler parentContextHandler, ContentValidations contentValidations, ContentValidation contentValidation) {
		super(parentContextHandler);

		this.contentValidation = contentValidation;
		this.contentValidations = contentValidations;
	}

	@Override
    public SaxContextHandler startElement(Attributes attributes, String uri, String localName, String qName) {
	    if(qName.equals("table:help-message")) {
	        final HelpMessage helpMessage = new HelpMessage(attributes);
	        contentValidation.setHelpMessage(helpMessage);
	        return new HelpMessageHandler(this, helpMessage);
	    }
	    else if(qName.equals("table:error-message")) {
	        final ErrorMessage errorMessage = new ErrorMessage(attributes);
	        contentValidation.setErrorMessage(errorMessage);
	        return new ErrorMessageHandler(this, errorMessage);
	    }
		final ElementNS child = new ElementNS(getFileDom(), attributes, uri, qName);
		contentValidation.getContent().add(child);
		return new UnknownContentHandler(this, child);
	}

    @Override
    public void endContext(String qName, String characters) {
        super.endContext(qName, characters);

        if(qName.equals("table:content-validation")) {
            if(contentValidation.getName()!=null&&!contentValidation.getName().isEmpty()) {
                contentValidations.addValidation(contentValidation, false);
            }
        }
    }
}
